import { Pie, Line, Area, Gauge, Column, Bar, RadialBar } from '@antv/g2plot'

const propAlias = '值'

import themeData from './themeData'

const chartHeight = 420
function checkStringLength(str) {
  const pattern = new RegExp('[\u4E00-\u9FA5]+')
  return pattern.test(str) ? 15 : 30
}

function defaultConfig(container, data, height, name, prop, extraData = {}) {
  height = height || chartHeight
  name = name || 'name'
  prop = prop || 'value'
  if (!document.getElementById(container)) {
    return new Promise((resolve, reject) => {
      reject('id不存在')
    })
  }
  return new Promise((resolve) => {
    resolve({ container, data, height, name, prop, extraData })
  })
}

function getAnnotations(dataType, label) {
  return [{
    type: 'text',
    position: ['max', dataType],
    content: label,
    offsetY: -4,
    style: {
      textBaseline: 'bottom'
    }
  },
  {
    type: 'line',
    start: ['start', dataType],
    end: ['end', dataType],
    style: {
      stroke: '#F4664A',
      lineDash: [2, 2]
    }
  }
  ]
}

export async function tableChart() {
  const res = await defaultConfig(...arguments)
  const { container, data, height, name, prop, extraData } = res
  const { chartOption = {} } = extraData
  const annotationsList = []
  if (chartOption.annotationList) {
    chartOption.annotationList.map(item => {
      annotationsList.push(...getAnnotations(item.type, item.label))
    })
  }
  const config = {
    data,
    xField: name,
    yField: prop,
    appendPadding: [40, 20, 20, 20],
    autoFit: true,
    height,
    label: {
      position: 'top'
    },
    xAxis: {
      label: {
        autoHide: false,
        autoRotate: true,
        formatter: (v) => {
          const strMaxLength = checkStringLength(v)
          if (v.length > strMaxLength) return v.substring(0, strMaxLength) + '...'
          else return v
        }
      }
    },
    meta: {},
    // tooltip: false,
    minColumnWidth: 20,
    maxColumnWidth: 20,
    annotations: [...annotationsList],
    ...chartOption
  }
  if (!config?.meta?.[prop]?.alias) {
    config.meta[prop] = {
      alias: extraData?.meta?.propAlias || propAlias
    }
  }
  // if (extraData.category) config.seriesField = extraData.category
  const columnPlot = new Column(container, config)
  columnPlot.update({ theme: themeData.theme })
  columnPlot.render()
  return columnPlot
}

tableChart.hasAnnotation = true

export async function pieSpiderChart() {
  const res = await defaultConfig(...arguments)
  const { container, data, height, name, prop, extraData } = res
  const { chartOption = {} } = extraData
  const piePlot = new Pie(container, {
    appendPadding: 10,
    data,
    height,
    angleField: prop,
    colorField: name,
    autoFit: true,
    radius: 0.75,
    label: {
      type: 'spider',
      labelHeight: 28,
      content: `{${name}}\n{${prop}}` // percentage比例
    },
    interactions: [{ type: 'element-selected' }, { type: 'element-active' }],
    ...chartOption
  })
  piePlot.update({ theme: themeData.theme })
  piePlot.render()
  return piePlot
}

export async function barChart() {
  const res = await defaultConfig(...arguments)
  let { container, data, height, name, prop, extraData } = res
  const { chartOption = {} } = extraData
  data = !extraData.clearSort ? data.sort((x, y) => y[prop] - x[prop]) : data

  const config = {
    data,
    height,
    xField: prop,
    yField: name,
    autoFit: true,
    appendPadding: [20, 40, 20, 20],
    label: {
      position: 'right'
    },
    meta: {},
    yAxis: {
      label: {
        formatter: (v) => {
          const strMaxLength = checkStringLength(v)
          if (v.length > strMaxLength) return v.substring(0, strMaxLength) + '...'
          else return v
        }
      }
    },
    minBarWidth: 20,
    maxBarWidth: 20,
    ...chartOption
  }
  if (!config?.meta?.[prop]?.alias) {
    config.meta[prop] = {
      alias: extraData?.meta?.propAlias || propAlias
    }
  }
  const barPlot = new Bar(container, config)
  barPlot.update({ theme: themeData.theme })
  barPlot.render()
  return barPlot
}

export async function lineChart() {
  const res = await defaultConfig(...arguments)
  const { container, data, height, name, prop, extraData } = res
  const { chartOption = {} } = extraData
  const annotationsList = []
  if (chartOption.annotationList) {
    chartOption.annotationList.map(item => {
      annotationsList.push(...getAnnotations(item.type, item.label))
    })
  }
  const config = {
    data,
    height,
    appendPadding: [20, 0, 0, 0],
    // padding: 40,
    xField: name,
    yField: prop,
    smooth: true,
    autoFit: true,
    // xAxis: {
    //   type: 'time',
    // },
    // yAxis: {
    //   label: {
    //     formatter: (v) => `${v}`.replace(/\d{1,3}(?=(\d{3})+$)/g, (s) => `${s},`)
    //   }
    // },
    smooth: true,
    legend: {
      offsetY: 20
    },
    animation: {
      appear: {
        animation: 'path-in',
        duration: 5000
      }
    },
    meta: {},
    annotations: [...annotationsList],
    ...chartOption
  }
  if (!config?.meta?.[prop]?.alias) {
    config.meta[prop] = {
      alias: extraData?.meta?.propAlias || propAlias
    }
  }
  if (extraData.category) config.seriesField = extraData.category
  if (extraData.xAxis) {
    if (extraData.xAxis === false) delete config.xAxis
    else config.xAxis = extraData.xAxis
  }
  const line = new Line(container, config)
  line.render()
  return line
}

export async function areaChart() {
  const res = await defaultConfig(...arguments)
  const { container, data, height, name, prop, extraData } = res
  const { chartOption = {} } = extraData
  const annotationsList = []
  if (chartOption.annotationList) {
    chartOption.annotationList.map(item => {
      annotationsList.push(...getAnnotations(item.type, item.label))
    })
  }
  const config = {
    data,
    height,
    autoFit: true,
    xField: name,
    yField: prop,
    xAxis: {
      range: [0, 1],
      tickCount: 5
    },
    meta: {},
    areaStyle: () => {
      return {
        fill: 'l(270) 0:#ffffff 0.5:#7ec2f3 1:#1890ff'
      }
    },
    annotations: [...annotationsList],
    ...chartOption
  }
  if (!config?.meta?.[prop]?.alias) {
    config.meta[prop] = {
      alias: extraData?.meta?.propAlias || propAlias
    }
  }
  if (extraData.category) config.seriesField = extraData.category
  const area = new Area(container, config)
  area.render()
  return area
}

export async function gaugeChart() {
  const res = await defaultConfig(...arguments)
  const { container, data, height, name, prop, extraData } = res
  const { chartOption = {} } = extraData
  const gauge = new Gauge(container, {
    percent: data[prop],
    height,
    autoFit: true,
    range: {
      color: '#30BF78'
    },
    indicator: {
      pointer: {
        style: {
          stroke: '#D0D0D0'
        }
      },
      pin: {
        style: {
          stroke: '#D0D0D0'
        }
      }
    },
    axis: {
      label: {
        formatter(v) {
          return Number(v) * 100
        }
      },
      subTickLine: {
        count: 3
      }
    },
    statistic: {
      content: {
        formatter: ({ percent }) => `${data[name]}: ${(percent * 100).toFixed(0)}%`,
        style: {
          color: 'rgba(0,0,0,0.65)',
          fontSize: 48
        }
      }
    },
    ...chartOption
  })
  gauge.render()
  return gauge
}

export async function radialBarChart() {
  const res = await defaultConfig(...arguments)
  const { container, data, height, name, prop, extraData } = res
  const { chartOption = {} } = extraData

  const bar = new RadialBar(container, {
    data,
    height,
    xField: name,
    yField: prop,
    autoFit: true,
    maxAngle: 270, // 最大旋转角度,
    radius: 0.8,
    innerRadius: 0.2,
    // barStyle: {
    //   lineCap: 'round',
    // },
    ...chartOption
  })
  bar.update({ theme: themeData.theme })
  bar.render()
  return bar
}

